/*  
  classMainWindow
  Developed on 2010 03 25
  Author R.G. care of Datalogic Automation Laser Marking Division
*/

/***********************************************************
  Internal Tables
***********************************************************/
const LINES_LOG=4;
const hIndex=0;
const getCurrentDistance=false;
var onlyTrigger=false;
const markWhenFocusReached="markWhenFocusReached";
const minimumVersion=[7, 1, 1];

const OK = "OK";
const WAIT = "Wait";
const WARNING="Warning";

const COM_ERROR_array = [(131072, "Sensor communication checksum ERROR"), (262144, "Sensor communication timeout ERROR"), (524288, "Sensor communication overrun ERROR"), (1048576, "Sensor communication frame ERROR"), (2097152, "Sensor communication invalid params "), (4194304, "Sensor communication unknown command ERROR")];

const SENSOR_ERROR_array = [1, 32, 64, 4, 65536];   //for each element of the array, the last element is the bitwise state when such error happens

const ParityMap = [ [ComPort.PAR_NONE, "None"],
    [ComPort.PAR_ODD, "Odd"],
    [ComPort.PAR_EVEN , "Even"],
    [ComPort.PAR_MARK , "Mark"],
    [ComPort.PAR_SPACE , "Space"] ];
const BaudsMap = [ [ComPort.BAUD110, "B 110"],
    [ComPort.BAUD300, "B 300"],
    [ComPort.BAUD600, "B 600"],
    [ComPort.BAUD1200, "B 1200"],
    [ComPort.BAUD2400, "B 2400"],
    [ComPort.BAUD4800, "B 4800"],
    [ComPort.BAUD9600, "B 9600"],
    [ComPort.BAUD14400, "B 14400"],
    [ComPort.BAUD19200, "B 19200"],
    [ComPort.BAUD38400, "B 38400"],
    [ComPort.BAUD56000, "B 56000"],
    [ComPort.BAUD57600, "B 57600"],
    [ComPort.BAUD115200, "B 115200"],
    [ComPort.BAUD128000, "B 128000"],
    [ComPort.BAUD256000, "B 256000"] ];
const DataBitsMap = [ [ComPort.DATA_5, "b 5"],
    [ComPort.DATA_6, "b 6"],
    [ComPort.DATA_7, "b 7"],
    [ComPort.DATA_8, "b 8"] ];
const StopBitsMap = [ [ComPort.STOP_1, "b 1"],
    [ComPort.STOP_1_5, "b 1.5"],
    [ComPort.STOP_2, "b 2"] ];
const FlowMap = [ [ComPort.FLOW_OFF, "Off"],
    [ComPort.FLOW_HARDWARE, "HW"],
    [ComPort.FLOW_XONXOFF, "Xon / Xoff"] ];

/***********************************************************
  Class classMainWindow Definition
***********************************************************/
classMainWindow.prototype = new Dialog("Datalogic S.r.l.", Dialog.D_NONE, false);

function classMainWindow(receiver)
{
    this.InReceiver = receiver;
    
      this.lbl = new Array(2);
  
  for (i = 0; i < this.lbl.length; ++i)
  {
    this.lbl[i] = new Label();
	this.lbl[i].alignment = 0x04;
    this.lbl[i].text = "";
  }
  this.lbl[0].text = AppTitle;
  
  this.gbMarkMode = new GroupBox();
  this.cbAutoManual = new CheckBox();
  this.cbAutoManual.text="Start Marking when Focal Distance is reached";
  this.cbAutoManual.sigStateChanged.connect(updateGlobalVariable);
  this.gbMarkMode.add(this.cbAutoManual);
  this.gbMarkMode.newColumn();
  this.pbManualMarking = new PushButton();
  this.pbManualMarking.setFixedSize(200,25);
  this.pbManualMarking.text="Start Marking";
  this.pbManualMarking.sigReleased.connect(onMark);
  this.gbMarkMode.add(this.pbManualMarking);
  this.add(this.gbMarkMode);
  
  var varState = System.getStringValue(markWhenFocusReached);
  if((varState.match!=("ERR")) && (varState==1)) {  //if the global variable exists and it is =1
      this.cbAutoManual.checked=true;
  }
  else this.cbAutoManual.checked=false;
  
  this.cmb = new Array(6);
  for (i = 0; i < this.cmb.length; ++i)
    this.cmb[i] = new ComboBox();
  
  this.cmb[0].label = "Port";
  var list = [];
  for (i=1; i<6; ++i)
    list.push("COM" + i);
  this.cmb[0].itemList = list;
  this.cmb[0].currentItem = h_port.instance.portName;
  
  this.cmb[1].label = "BaudRate";
  var list = [];
  for (i=0; i<BaudsMap.length; ++i)
    list.push(BaudsMap[i][1]);
  this.cmb[1].itemList = list;
  this.cmb[1].currentItem = lookUp(BaudsMap, h_port.instance.baudRate);
  
  this.cmb[2].label = "Data Bits";
  var list = [];
  for (i=0; i<DataBitsMap.length; ++i)
    list.push(DataBitsMap[i][1]);
  this.cmb[2].itemList = list;
  this.cmb[2].currentItem = lookUp(DataBitsMap, h_port.instance.dataBits);
  
  this.cmb[3].label = "Parity";
  var list = [];
  for (i=0; i<ParityMap.length; ++i)
    list.push(ParityMap[i][1]);
  this.cmb[3].itemList = list;
  this.cmb[3].currentItem = lookUp(ParityMap, h_port.instance.parity);
  
  this.cmb[4].label = "Stop";
  var list = [];
  for (i=0; i<StopBitsMap.length; ++i)
    list.push(StopBitsMap[i][1]);
  this.cmb[4].itemList = list;
  this.cmb[4].currentItem = lookUp(StopBitsMap, h_port.instance.stopBits);
  
  this.cmb[5].label = "Flow Control";
  var list = [];
  for (i=0; i<FlowMap.length; ++i)
    list.push(FlowMap[i][1]);
  this.cmb[5].itemList = list;
  this.cmb[5].currentItem = lookUp(FlowMap, h_port.instance.flowControl);
    
    this.gbLayout = new GroupBox();
    this.gbLayout.title="Layout selection";
    this.cbDocs = new ComboBox();
    this.cbDocs.label="Layout: ";
    DocsDir=new Dir (System.getDocumentsDir());
    this.cbDocs.itemList = DocsDir.entryList("*.xlp");
    this.cbDocs["sigIndexChanged(QString)"].connect(this, "onIndexChanged");
    this.gbLayout.add(this.cbDocs);
    
      this.btn = new Array(2);
  for (i = 0; i < this.btn.length; ++i)
  {
    this.btn[i] = new PushButton();
	this.btn[i].setFixedSize(120,20);
	//this.add(this.btn[i]);
  }
  this.btn[0].text = "Open";
  this.btn[1].text = "Close";
    
    this.prv = new RenderArea();
    this.gbLayout.add(this.prv);
    
    this.add(this.gbLayout);
    
    this.msgLabel0 = new Label();
    this.msgLabel1 = new Label();
    this.msgLabel2 = new Label();
    this.msgLabel3 = new Label();
    this.msgLabel4 = new Label();
    this.lbTcpPort = new Label();
    
    this.msgLabel0.alignment = 0x04;
    this.msgLabel1.alignment = 0x04;
    this.msgLabel2.alignment = 0x04;
    this.msgLabel3.alignment = 0x04;
    this.msgLabel4.alignment = 0x04;
    this.lbTcpPort.alignment = 0x04;
    
    this.msgLabel0.text = "";
    this.msgLabel1.text = "";
    this.msgLabel2.text = "";
    this.msgLabel3.text = "";
    this.msgLabel4.text = AppTitle;    
    this.lbTcpPort.text = "";
    
    this.GroupBase = new GroupBox();
  /*this.GroupBase.title = "COM parameters";   //adding the serial communication settings to the GUI
  this.GroupBase.addSpace(5);
  this.GroupBase.add(this.cmb[0]);
  this.GroupBase.add(this.cmb[1]);
  this.GroupBase.add(this.cmb[2]);
  this.GroupBase.newColumn();
  this.GroupBase.addSpace(5);
  this.GroupBase.add(this.cmb[3]);
  this.GroupBase.add(this.cmb[4]);
  this.GroupBase.add(this.cmb[5]);*/
    this.GroupBase.title = "Outcome";
    this.lblResult = new Label();
  
    this.add(this.GroupBase);
    this.GroupBase.add(this.lblResult);
    this.GroupBase.newColumn();
    this.lblText = new Label();
    this.GroupBase.add(this.lblText);
    
    this.gbLastReport = new GroupBox();
    this.gbLastReport.title= "Last acquisition";
    this.add(this.gbLastReport);
    this.leDistance = new LineEdit();
    this.leDistance.label = "Distance [mm]: "
    this.leDistance.enable=false;
    this.gbLastReport.add(this.leDistance);
    this.gbLastReport.newColumn();
    this.PBtrigger = new PushButton;
    this.PBtrigger.text="TRIGGER";
    this.PBtrigger.setFixedSize(200,25);
    this.PBtrigger["sigReleased()"].connect(this, "onTrigger");
    this.gbLastReport.add(this.PBtrigger);
    
    
    this.gbButtons = new GroupBox();
    this.PBsaveDistance = new PushButton;
    this.PBsaveDistance.text="SAVE CURRENT DISTANCE";
    this.PBsaveDistance.setFixedSize(200,50);
    this.PBsaveDistance["sigReleased()"].connect(this, "onSaveDistance");
    this.gbButtons.add(this.PBsaveDistance);
    this.gbButtons.newColumn();
    this.PBfocusStart = new PushButton;
    this.PBfocusStart.text="START AUTOFOCUS";
    this.PBfocusStart.setFixedSize(200,50);
    this.PBfocusStart["sigReleased()"].connect(this, "onFocusStart");
    this.PBfocusStart.enable=false;
    this.gbButtons.add(this.PBfocusStart);
    this.add(this.gbButtons);
    
    this.lblLog = new Array(LINES_LOG);
    for (var i=0;i<this.lblLog.length;++i) {
	this.lblLog[i] = new Label();
	this.add(this.lblLog[i]);
    }
    this.show();
}

classMainWindow.prototype.showMessage = function( target, message )
{
  this.lbl[target].text = message;
}

classMainWindow.prototype.onIndexChanged = function () {
    print(this.cbDocs.itemList.length +" files detected")
	    print("result: "+hDoc.load(this.cbDocs.currentItem));
    var ids = hDoc.getObjectIDs().split(",");
    print("ids: "+ids);
    this.prv.preview(hDoc);
}

function updateGlobalVariable(state) {
    print("updateGlobalVariable - state:"+state);
    if (state==2) {   //the checkbox has just been checked
	System.makeStringVariable(markWhenFocusReached, "1", true);
    }
    else {   //the checkbox has been disabled
	System.makeStringVariable(markWhenFocusReached, "0", true);
    }
}

classMainWindow.prototype.onFocusStart = function() {    
    print("onFocusStart");
    autofocusExecuting=true;
    this.leDistance.text=" ";
    /*START ADDITIONAL CODE part 3 - Implementation task #78612*/
    if(!this.checkErrors()) {
	var s = Axis.autofocus();
	if(s) {
	    this.Wait();
	    log("Autofocus started successfuly");
	}
	else {
	    this.Warning();
	    log("Autofocus cannot be started");
	    autofocusExecuting=false;
	}
    }
    /*END ADDITIONAL CODE part 3 - Implementation task #78612*/
}

function checkLighterVersion(currentVersion) {
    if (currentVersion[0]>minimumVersion[0]) return true;  //major release version
    else {
	if (currentVersion[1]>=minimumVersion[1]) return true;  //minor release version 
	else {
	    if (currentVersion[2]>=minimumVersion[2]) return true;   //build release version
	    else {
		var LighterErrorBox = MessageBox.information("This version of Lighter does not support the Distance Sensor functionalities - available from Lighter Suite v. "+minimumVersion);
		return false;
	    }
	}
    }
    
}

function log (message) {   //this function organizes the messages on the Dialog window
   var d = new Date();
   message = d.toLocaleTimeString() +" - " + message;   
   if(hIndex<LINES_LOG) {
       h_Window.lblLog[hIndex].text=message;  
       hIndex++;
   }
   else {
       for (var j = 1; j < LINES_LOG; ++j)   //label shift cycle upwards
       h_Window.lblLog[j - 1].text = h_Window.lblLog[j].text;
       h_Window.lblLog[h_Window.lblLog.length - 1].text = message;
   }
   print(message);
}

classMainWindow.prototype.onSaveDistance = function() {
    /*getCurrentDistance=true;
    savedPosition=Axis.getPosition(2);
    h_Proto.protocolSend("{0M}");
    this.PBfocusStart.enable=true;*/
    
    /*START ADDITIONAL CODE part 2 - Implementation task #78612*/
    if(!this.checkErrors()) {   //If sensor is OUT OF RANGE
	h_Window.OK();
	var s = Axis.setSensorDistanceReference();
	print("Axis.setSensorDistanceReference() returned: "+s);
	var t= Axis.getCurrentSensorDistance().toFixed(2);
	this.leDistance.text=t;
	if (s) log("Current Distance "+t+" saved as reference");
	else log("Current Distance cannot be saved as reference");
	this.PBfocusStart.enable=true;
    }
    /*END ADDITIONAL CODE part 2 - Implementation task #78612*/
}

classMainWindow.prototype.OK = function () {
    this.lblResult.loadImage(System.getProjectDir() + "/" + OK,20,20,2);
    this.lblText.text="AXIS IN POSITION";
}

classMainWindow.prototype.Wait = function () {
    this.lblResult.loadImage(System.getProjectDir() + "/" + WAIT,20,20,2);
    this.lblText.text="MOVING AXIS INTO POSITION";
}

classMainWindow.prototype.WarningMex = function (mex) {
    this.lblResult.loadImage(System.getProjectDir() + "/" + WARNING,20,20,2);
    this.lblText.text=mex;
}


classMainWindow.prototype.onTrigger = function() {
    print("Status: "+Axis.getDistanceSensorStatus());
    /*START ADDITIONAL CODE part 1 - Implementation task #78612*/
    if((this.checkErrors())||(this.checkReference())) {   //report error
	//h_Window.OutOfRange();
	this.leDistance.text="-";
    }
        /*END ADDITIONAL CODE part 1 - Implementation task #78612*/
    else  {
	h_Window.OK();
	var t = Axis.getCurrentSensorDistance();
	print("Axis.getCurrentSensorDistance() returned: "+t);
	this.leDistance.text=t.toFixed(2);
    }
}

classMainWindow.prototype.checkErrors = function() {
    print("Status: "+Axis.getDistanceSensorStatus());
    var errorPresence=false;
    for (i=0;i<COM_ERROR_array.length;i++) {
	if(Axis.getDistanceSensorStatus() & COM_ERROR_array[0][i]) {
	    print("COMMUNICATION ERROR: "+COM_ERROR_array[1][i]);
	    errorPresence=true;
	}
    }
    
    if((Axis.getDistanceSensorStatus() & SENSOR_ERROR_array[0])==false) {
	print("Error 1");
	this.WarningMex("Focus Sensor is not Enabled");
	this.leDistance.text="-";
	errorPresence=true;
    }
    else if (Axis.getDistanceSensorStatus() & SENSOR_ERROR_array[1]) {
	print("Error 2");
	this.WarningMex("Focus Sensor out of range");
	this.leDistance.text="-";
	errorPresence=true;
    }
    else if((Axis.getDistanceSensorStatus() & SENSOR_ERROR_array[4])==false) {
	print("Error 3");
	this.WarningMex("Focus Sensor connection error");
	this.leDistance.text="-";
	errorPresence=true;
    }
    print("errorPresence: "+errorPresence);
    if(!errorPresence) return false;   //if communication errors are NOT present
    else return true;    //if communication errors are present
}

classMainWindow.prototype.checkReference = function() {
    print("Status: "+Axis.getDistanceSensorStatus());
    var errorPresence=false;
    if(Axis.getDistanceSensorStatus() & SENSOR_ERROR_array[2]) {
	print("Error 3");
	this.WarningMex("Invalid Focus reference");
	this.leDistance.text="-";
	errorPresence=true;
    }
    else if((Axis.getDistanceSensorStatus() & SENSOR_ERROR_array[3])==false) {
	print("Error 4");
	this.WarningMex("Distance reference not configured");
	this.leDistance.text="-";
	errorPresence=true;
    }
    print("errorPresence: "+errorPresence);
    if(!errorPresence) return false;   //if errors are NOT present
    else return true;    //if errors are present
}
